/*
Copyright (c) 2004 EarthLink, Inc.
All rights reserved.

FPeer Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* FSPclient.java
 * 
 * Created on Apr 27, 2004
 *
 * 
*/
package net.earthlink.research.fsp;

import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;

import java.util.Observer;
import java.util.Observable;

import java.util.Enumeration;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.ConsoleAppender;

/**
 * @author petrovic
 *
 */
public class FSPclient extends Observable implements Observer {

   private int        port;
   private String     address;   
   private FSPsession session;
   
   private Logger          logger;
   private PatternLayout   layout;
   private ConsoleAppender appender;

   public FSPclient(String address, int port) {
      this.address   = address;            
      this.port      = port;
      
      logger   = Logger.getLogger(this.getClass().getPackage().getName());
      
      // Somewhat lame protection against adding redundant appenders...
      
      Enumeration e = logger.getAllAppenders();
      if( ! e.hasMoreElements() ) {
         layout   = new PatternLayout("%d{dd MMM yyyy HH:mm:ss}, %l, %m %n");
         appender = new ConsoleAppender(layout);
         logger.addAppender(appender);
         logger.setLevel(Level.WARN);
      }      
   }
   
   public void download(String fileName, String outputDirectory, String saveAs) throws IOException {
      FileOutputStream fileOutputStream;
      
      session          = new FSPsession(InetAddress.getByName(address), port);
      
      logger.warn("My FSP client wants <" + address + ":" + Integer.toString(port) + "/" + fileName + "=" + saveAs + ">");      
      
      fileOutputStream = new FileOutputStream(outputDirectory + "/" + saveAs);
      FSPutil util = new FSPutil();
      util.addObserver(this);
      //FSPutil.download(session, fileName, fileOutputStream, 0);      

      try {
         util.download(session, fileName, fileOutputStream, 0);      
      }
      catch (IOException e) {
         throw e;
      }
      finally {
         fileOutputStream.close();
         session.close();
      }
   }

   public void update(Observable observable, Object o) {
      setChanged();
      notifyObservers(o);
   }
}
