/*
Copyright (c) 2004 EarthLink, Inc.
All rights reserved.

FPeer Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Subscribers.java

package net.earthlink.research.p2p;

import java.util.Iterator;
import java.util.Vector;

import javax.sip.address.*;

import org.apache.log4j.Logger;


/**
*
* @author Mark Petrovic
*/
public class Subscribers extends Vector {

   private Logger logger;

   public Subscribers() {
      logger = Logger.getLogger(this.getClass().getName());
   }

   /**
    * 
    * @param s
    */
   public void add (Subscriber s) {
      
      /* 

      A peer can subscribe to our peerlist, quit its runtime client
      without notice, then reappear later and subscribe again during
      our same runtime invocation.  When this happens, we must not add
      the peer to our Subscriber list without removing the old instance,
      bc the old instance contains now-stale SIP Dialog state.

      Similarly, remove it from our Subscriptions, and add the host back
      to NetherHosts.

      */

      if( isSubscriber(s.getHost()) ) {
         this.remove( getByIPAddress(s) );         

         FPeer.fpeer.subscriptions.remove(s.getHost());
         FPeer.fpeer.netherHosts.add(s.getHost());
         logger.warn("Subscriber replaced = " + s.getHost());
      }
      else 
         logger.warn("Subscriber added = " + s.getHost());
      
      super.add(s);  
      FPeer.fpeer.updateWindowManager();
   }

   /**
    * 
    * @param sub
    */
   public void remove (Subscriber sub) {
      Subscriber s;
      Vector     v;
    
      v = new Vector();
      for(Iterator i = iterator(); i.hasNext(); ) {
         s = (Subscriber )i.next();
         if( sub.getHost().equals(s.getHost())) {
            v.add(sub);
         }
      }
   
      /* Using buffer v allows us to remove elements from
      "this" without iterating over "this", which can produce
      java.util.ConcurrentModificationException.  */
  
      for( Iterator j = v.iterator(); j.hasNext(); ) {
         Subscriber subscriber;

         subscriber = (Subscriber )j.next();
         super.remove(subscriber);
         logger.warn("Subscriber removed = " + subscriber.getHost());
      }
      FPeer.fpeer.updateWindowManager();
   }

   //-----------------------

   /**
    * 
    * @param sub
    */
   public void remove (String host) {
      Subscriber s;
      Vector     v;
      
      v = new Vector();
      for(Iterator i = iterator(); i.hasNext(); ) {
         s = (Subscriber )i.next();
         if( s.getHost().equals(host)) {
            v.add(s);
         }
      }

      FPeer.fpeer.subscriptions.remove(host);
      //FPeer.fpeer.netherHosts.add(host);   /// Hmmm.
     
      /* Using buffer v allows us to remove elements from
      "this" without iterating over "this", which can produce
      java.util.ConcurrentModificationException.  */
    
      for( Iterator j = v.iterator(); j.hasNext(); ) {
         Subscriber h;

         h = (Subscriber )j.next();
         super.remove(h);
         logger.warn("Subscriber removed = " + h.getHost());
      }
      FPeer.fpeer.updateWindowManager();
   }
   
   //-----------------------

   /**
    * 
    * @param uri
    * @return
    */
   public Subscriber getByIPAddress(Subscriber s) {
      Subscriber subscriber;
      String     keyHost; 
      
      subscriber = null;
      keyHost    = s.getHost();

      for( Iterator i = iterator(); i.hasNext(); ) {
         subscriber = (Subscriber )i.next();
         if (subscriber.getHost().equals(keyHost))
            break;
      }
      return subscriber;
   }

   /**
    * 
    * @param uri
    * @return
    */
   public boolean isSubscriber(SipURI uri) {
      boolean    isOnList;
      Subscriber subscriber;
      //
      isOnList = false;
      for( Iterator i = iterator(); i.hasNext(); ) {
         subscriber = (Subscriber )i.next();
         isOnList   = isOnList || subscriber.equals(uri);
         if (isOnList)
            break;
      }
      return isOnList;
   }

   /**
    * 
    * @param address
    * @return
    */
   public boolean isSubscriber(String address) {
      boolean    isOnList;
      Subscriber subscriber;

      isOnList = false;
      for( Iterator i = iterator(); i.hasNext(); ) {
         subscriber = (Subscriber )i.next();
         isOnList   = isOnList || subscriber.equalsHost(address);
         if (isOnList)
            break;
      }
      return isOnList;
   }

   
}
